@@ -12,16 +12,19 @@
 #include <R_ext/BLAS.h>
 #include <R_ext/Lapack.h>
 #include <R_ext/Applic.h>
-#include "ezprof.h"
-#include "zeroin.h"
+//extern "C" {
+	#include "ezprof.h"
+	#include "zeroin.h"
+//}
 
 struct ctrl_fanc_t {
 	double tol_EM, tol_CD, tol_Psi, tol_Phi, eta, zita, delta;
-	double *init_coef, max_rho, max_gamma, min_gamma, gamma_ebic;
+	double *init_coef, *rhos, *gammas, *max_rho, max_gamma, min_gamma, gamma_ebic, *w;
 	int num_rhos, num_gammas;
+	double alpha_powerseq;
 	int maxit_EM, maxit_CD, maxit_Phi;
 	int cor_factor, use_penalty, zero_min_rho;
-	int cold_init, num_iter_init, p_max_for_S;
+	int cold_init, num_iter_init, num_iter_init_prenet, p_max_for_S;
 	int progress, omp, num_threads;
 };
 
@@ -160,6 +163,37 @@ static double logseq(int n, double a, double b, double *arr) {
 }
 
 
+static double h(double x, double alpha) {
+	//return alpha^(-1.0) * C^(1.0-alpha) * ((x + C)^alpha - C^alpha);
+	double ans;
+	if(alpha == 0.0) ans = log(x);
+	if(alpha != 0.0) ans = pow(x, alpha);
+	return ans;
+}
+static double hinv(double y, double alpha) {
+	double ans;
+	if(alpha == 0.0) ans = exp(y);
+	if(alpha != 0.0) ans = exp(log(y)/alpha);
+	return ans;
+}
+
+static double powerseq(int n, double a, double b, double alpha, double *arr) {
+	double h_a = h(a, alpha), h_b = h(b, alpha);
+	for (int i = 0; i < n; i++) {
+		arr[i] = hinv((h_b - h_a) * i / (n - 1) + h_a, alpha);
+	}
+	return 0; 
+}
+
+
+static double seq(int n, double a, double b, double *arr) {
+	for (int i = 0; i < n; i++) {
+		arr[i] = (b-a) * i / (n - 1) + a;
+	}
+	return 0; 
+}
+
+
 // ---------------------------------------------------------
 //  solution for MC+ in one dimension
 // ---------------------------------------------------------
@@ -212,6 +246,13 @@ double S_MC(double theta, double rho, double gamma) {
 	}
 }
 
+
+
+double S_lasso(double theta, double rho){
+	return dsignum(theta) * dmax((fabs(theta) - rho), 0);
+}
+
+
 // ---------------------------------------------------------
 //  Update Phi using BFGS
 // ---------------------------------------------------------
@@ -458,7 +499,7 @@ static void update_Lambda_i(
 	
 	for (int j = 0; j < m; j++) {
 		Lambda_i[j] = Lambda[i + j * p];
-		Lambda_i_new[j] = 0;
+		Lambda_i_new[j] = Lambda[i + j * p];
 	}
 	
 	// Minimize w.r.t. Lambda[i, ] with coordinate descent
@@ -469,6 +510,8 @@ static void update_Lambda_i(
 			double sum_A_Lambda = 0;
 			for (int k = 0; k < m2; k++) {
 				double lamb = k < j ? Lambda_i_new[k] : k > j ? Lambda_i[k] : 0;
+        		//another code:
+				//double lamb = k != j ? Lambda_i_new[k] : 0;
 				sum_A_Lambda += A[k + j * m] * lamb;
 			}
 			double z = (B[j + i * m] - sum_A_Lambda) / A[j + j * m];
@@ -481,9 +524,12 @@ static void update_Lambda_i(
 		
 		// Check convergence
 		err_CD = dnorm2(m2, Lambda_i_new, Lambda_i);
+		err_CD = err_CD / sqrt(m); //changed the threshold!!
 		
 		// Step
 		SWAP(double*, Lambda_i, Lambda_i_new);
+        //another code:
+        //F77_CALL(dcopy)(&m2, Lambda_i_new, INC_SEQ, Lambda_i, INC_SEQ);
 		iter_CD++;
 	}
 	
@@ -493,6 +539,64 @@ static void update_Lambda_i(
 	*O_conv |= err_CD > ctrl->tol_CD ? 1 : 0;
 }
 
+
+
+static void update_Lambda_i_prenet(
+	int p, int m, int m2, int i, double *Lambda, double *diag_Psi,
+	double *A, double *B, double rho, double gamma, struct ctrl_fanc_t *ctrl,
+	double *O_Lambda_new, int *O_conv)
+{
+	double Lambda_i_buf1[m], Lambda_i_buf2[m];
+	double *Lambda_i = Lambda_i_buf1, *Lambda_i_new = Lambda_i_buf2;
+	
+	for (int j = 0; j < m; j++) {
+		Lambda_i[j] = Lambda[i + j * p];
+		Lambda_i_new[j] = Lambda[i + j * p];
+	}
+
+			// Minimize w.r.t. Lambda[i, ] with coordinate descent
+	double err_CD = DBL_MAX;
+	int iter_CD = 0;
+	while (err_CD > ctrl->tol_CD && iter_CD < ctrl->maxit_CD) {
+		for (int j = 0; j < m2; j++) {
+			double sum_A_Lambda = 0;
+			double sum_absLambda = 0;
+			double sum_Lambda2 = 0;
+			for (int k = 0; k < m2; k++) {
+				double lamb = k < j ? Lambda_i_new[k] : k > j ? Lambda_i[k] : 0;
+        		//another code:
+				//double lamb = k != j ? Lambda_i_new[k] : 0;
+				sum_A_Lambda += A[k + j * m] * lamb;
+				sum_absLambda += fabs(lamb);
+				sum_Lambda2 += lamb * lamb;
+			}
+			double beta_prenet = rho * diag_Psi[i] * (1 - gamma) * sum_Lambda2 * ctrl->w[i] * ctrl->w[i];
+			double gamma_prenet = gamma * sum_absLambda * ctrl->w[i];
+			double z = (B[j + i * m] - sum_A_Lambda) / (A[j + j * m] + beta_prenet);
+			double wrho = diag_Psi[i] * rho * gamma_prenet / (A[j + j * m] + beta_prenet);
+			
+			Lambda_i_new[j] = S_lasso(z,wrho);
+		}
+		
+		// Check convergence
+		err_CD = dnorm2(m2, Lambda_i_new, Lambda_i);
+		err_CD = err_CD / sqrt(m); //changed the threshold!!
+
+		// Step
+		SWAP(double*, Lambda_i, Lambda_i_new);
+        //another code:
+        //F77_CALL(dcopy)(&m2, Lambda_i_new, INC_SEQ, Lambda_i, INC_SEQ);
+		iter_CD++;
+	}
+	
+	for (int j = 0; j < m; j++) {
+		O_Lambda_new[i + j * p] = Lambda_i[j];
+	}
+	*O_conv |= err_CD > ctrl->tol_CD ? 1 : 0;
+}
+
+
+
 static void update_Psi(
 	int p, int m, double *Lambda, double *A, double *B, double *S,
 	struct ctrl_fanc_t *ctrl,
@@ -519,7 +623,7 @@ static void update_Psi(
 
 static void calculate_logF(
 	int p, int m, int N, double *Lambda, double *diag_Psi, double *Phi,
-	double *S, double *X, double *Im, double rho, double gamma,
+	double *S, double *X, double *Im, double rho, double gamma, int type,
 	int cor_factor, struct ctrl_fanc_t *ctrl,
 	struct logF_value_t *O_logF)
 {
@@ -536,7 +640,7 @@ static void calculate_logF(
 	F77_CALL(dgemm)(
 		TRANS_N, TRANS_N, &m, &m, &m, &D_ONE, M, &m, A, &m, &D_ZERO, MA, &m);
 	
-	// Calculate log-likelihood
+	// Calculate log-likelihood (log(det(Sigma)) + tr(Sigma^{-1}S))
 	double logF = 0;
 	for (int i = 0; i < p; i++) {
 		logF += log(diag_Psi[i]) + S[i + i * p] / diag_Psi[i];
@@ -549,18 +653,29 @@ static void calculate_logF(
 	
 	// Calculate penalty
 	double penalty = 0;
-	for (int i = 0; i < pm; i++) {
-		if (rho == 0) {
-			// pass
-		} else if (fabs(Lambda[i]) < rho * gamma) {
-			penalty += rho *
-				(fabs(Lambda[i]) - Lambda[i] * Lambda[i] / (2 * rho * gamma));
-		} else {
-			penalty += rho * rho * gamma / 2;
+	if(type==1){
+			for (int i = 0; i < pm; i++) {
+			if (rho == 0) {
+				// pass
+			} else if (fabs(Lambda[i]) < rho * gamma) {
+				penalty += rho *
+					(fabs(Lambda[i]) - Lambda[i] * Lambda[i] / (2 * rho * gamma));
+			} else {
+				penalty += rho * rho * gamma / 2;
+			}
+		}
+		for (int i = 0; i < p; i++) {
+			penalty += ctrl->eta * S[i + i * p] * (1 / diag_Psi[i]) / 2;
+		}
+	}else{
+		for (int i = 0; i < p; i++){
+			for (int k = 0; k < m-1; k++){
+				for (int l = k+1; l < m; l++){
+					penalty += rho * 1/2 * (1-gamma) * ctrl->w[i] * ctrl->w[i] *  Lambda[i+k*p] * Lambda[i+k*p] * Lambda[i+l*p] * Lambda[i+l*p];
+					penalty += rho  * gamma * ctrl->w[i] * fabs(Lambda[i+k*p] * Lambda[i+l*p]);
+				}
+			}
 		}
-	}
-	for (int i = 0; i < p; i++) {
-		penalty += ctrl->eta * S[i + i * p] * (1 / diag_Psi[i]) / 2;
 	}
 	
 	O_logF->logF = logF;
@@ -578,7 +693,7 @@ static void minimize_logF_Lambda_Psi_Phi(
 	int p, int m, int N, int m2_init,
 	double *Lambda_init, double *diag_Psi_init, double *Phi_init,
 	double *S, double *X, double *Im,
-	double rho, double gamma,
+	double rho, double gamma, int type, int init_prenet,
 	int cor_factor, struct ctrl_fanc_t *ctrl,
 	int *O_m2, double *O_Lambda, double *O_diag_Psi, double *O_Phi,
 	struct logF_value_t *O_logF, int *O_conv)
@@ -591,11 +706,18 @@ static void minimize_logF_Lambda_Psi_Phi(
 	double *diag_Psi_new = (double *)R_alloc(p, sizeof(double));
 	double *Phi = (double *)R_alloc(mm, sizeof(double));
 	double *Phi_new = (double *)R_alloc(mm, sizeof(double));
+	double *Phi_tmp = (double *)R_alloc(mm, sizeof(double));
 	double *A = (double *)R_alloc(mm, sizeof(double));
 	double *inv_A = (double *)R_alloc(mm, sizeof(double));
 	double *B = (double *)R_alloc(pm, sizeof(double));
 	double *M = (double *)R_alloc(mm, sizeof(double));
 	
+    double *diagAinv = (double *)R_alloc(m, sizeof(double));
+    double *comparison_mv = (double *)R_alloc(pm, sizeof(double));
+    int *index_comparison_mv = (int *)R_alloc(p, sizeof(int));
+    int *index_nonzerocolumn = (int *)R_alloc(m, sizeof(int));
+    int *index_zerocolumn = (int *)R_alloc(m, sizeof(int));
+
 	// Initialize
 	F77_CALL(dcopy)(&pm, Lambda_init, INC_SEQ, Lambda, INC_SEQ);
 	F77_CALL(dcopy)(&p, diag_Psi_init, INC_SEQ, diag_Psi, INC_SEQ);
@@ -604,6 +726,8 @@ static void minimize_logF_Lambda_Psi_Phi(
 	// Iterate EM step
 	double err_Lambda = DBL_MAX;
 	int iter_EM = 0;
+
+
 	while (err_Lambda > ctrl->tol_EM && iter_EM < ctrl->maxit_EM) {
 		// Prepare
 		ezprof_start(g_ezprof, 0, "Calculate_M_A_B");
@@ -613,31 +737,74 @@ static void minimize_logF_Lambda_Psi_Phi(
 		ezprof_stop(g_ezprof, 0);
 		
 		// Update Lambda
-		ezprof_start(g_ezprof, 1, "Update Lambda");
-		if (ctrl->use_penalty) {
-			// For each coord
-#ifdef _OPENMP
-#pragma omp parallel for num_threads(ctrl->num_threads) if(ctrl->omp)
-#endif
-			for (int i = 0; i < p; i++) {
-				update_Lambda_i(
-					p, m, m2, i, Lambda, diag_Psi, A, B, rho, gamma, ctrl,
-					Lambda_new, &O_conv[1]);
-			}
-		} else {
-			// Calculate inv(A)
-			inv(m, A, inv_A, NULL);
-			
-			// Calculate Lambda <- t(B) %*% inv(A)
-			F77_CALL(dgemm)(TRANS_T, TRANS_N, &p, &m, &m,
-				&D_ONE, B, &m, inv_A, &m,
-				&D_ZERO, Lambda_new, &p);
-			for (int i = m2 * p; i < pm; i++) {
-				Lambda_new[i] = Lambda_init[i];
+		if(init_prenet==0){
+			ezprof_start(g_ezprof, 1, "Update Lambda");
+			if (ctrl->use_penalty) {
+				// For each coord
+	#ifdef _OPENMP
+	#pragma omp parallel for num_threads(ctrl->num_threads) if(ctrl->omp)
+	#endif
+				for (int i = 0; i < p; i++) {
+					if(type==1) update_Lambda_i(
+						p, m, m2, i, Lambda, diag_Psi, A, B, rho, gamma, ctrl,
+						Lambda_new, &O_conv[1]);
+
+					if(type==2) update_Lambda_i_prenet(
+						p, m, m2, i, Lambda, diag_Psi, A, B, rho, gamma, ctrl,
+						Lambda_new, &O_conv[1]);
+				}
+
+			} else {
+				// Calculate inv(A)
+				inv(m, A, inv_A, NULL);
+				
+				// Calculate Lambda <- t(B) %*% inv(A)
+				F77_CALL(dgemm)(TRANS_T, TRANS_N, &p, &m, &m,
+					&D_ONE, B, &m, inv_A, &m,
+					&D_ZERO, Lambda_new, &p);
+				for (int i = m2 * p; i < pm; i++) {
+					Lambda_new[i] = Lambda_init[i];
+				}
 			}
+			ezprof_stop(g_ezprof, 1);
+		}else{
+			//diagAinv <- diag(A^(-1))
+	        for (int i = 0; i < m; i++) {
+	            diagAinv[i] = 1.0 / A[i + i*m];
+	        }
+
+	        //comparison_mv <- diagAinv * B^2
+	        for (int i = 0; i < p; i++) {
+	            for (int j = 0; j < m; j++) {
+	                comparison_mv[j+i*m] = diagAinv[j] * B[j+i*m] * B[j+i*m];
+	            }
+	        }
+
+	        //index <- apply(comparison_mv,2,which.max)
+	        for (int i = 0; i < p; i++) {
+	            double tmp=0.0;
+	            for (int j = 0; j < m; j++) {
+	                if(comparison_mv[j+i*m] > tmp){
+	                    index_comparison_mv[i] = j;
+	                    tmp = comparison_mv[j+i*m];
+	                }
+	            }
+	        }
+
+	        //Lambdanew_all <- diagAinv * B
+	        //Lambdanew[] <- 0
+	        //for(i in 1:p) Lambdanew[i,index[i]] <- Lambdanew_all[index[i],i]
+	         for (int i = 0; i < p; i++) {
+	            for (int j = 0; j < m; j++) {
+	                if(index_comparison_mv[i] == j){
+	                    Lambda_new[i+j*p] = diagAinv[j] * B[j+i*m];
+	                }else{
+	                    Lambda_new[i+j*p] = 0.0;
+	                }
+	            }
+	        }
 		}
-		ezprof_stop(g_ezprof, 1);
-		
+
 		// Update Psi
 		ezprof_start(g_ezprof, 2, "Update Psi");
 		update_Psi(p, m, Lambda_new, A, B, S, ctrl, diag_Psi_new);
@@ -652,27 +819,66 @@ static void minimize_logF_Lambda_Psi_Phi(
 		}
 		ezprof_stop(g_ezprof, 3);
 		
-		// Swap zero columns in Lambda backward
-		int j2 = 0;
-		for (int j = 0; j < m2; j++) {
-			double sum_abs_Lambda_j = 0;
-			for (int i = 0; i < p; i++) {
-				sum_abs_Lambda_j += fabs(Lambda_new[i + j * p]);
-			}
-			if (sum_abs_Lambda_j > ctrl->tol_EM) {
-				if (j2 < j) {
-					F77_CALL(dcopy)(
-						&p, &Lambda_new[j * p], INC_SEQ,
-						&Lambda_new[j2 * p], INC_SEQ);
-				}
-				j2++;
-			}
-		}
-		dfill(pm - j2 * p, Lambda_new + j2 * p, 0);
-		m2 = j2;
-		
+		 // Swap zero columns in Lambda backward
+        int j2 = 0;
+        int j2_2 = 0;
+        ifill(m, index_nonzerocolumn, -1);
+        ifill(m, index_zerocolumn, -1);
+        for (int j = 0; j < m2; j++) {
+            double sum_abs_Lambda_j = 0;
+            for (int i = 0; i < p; i++) {
+                sum_abs_Lambda_j += fabs(Lambda_new[i + j * p]);
+            }
+            if (sum_abs_Lambda_j > 0.0) {
+                index_nonzerocolumn[j2] = j;
+                if (j2 < j) {
+                    F77_CALL(dcopy)(
+                        &p, &Lambda_new[j * p], INC_SEQ,
+                        &Lambda_new[j2 * p], INC_SEQ);
+                }
+                j2++;
+            }else{
+                index_zerocolumn[j2_2] = j;
+                j2_2++;
+            }
+        }
+        dfill(pm - j2 * p, Lambda_new + j2 * p, 0);
+
+       int j = 0;
+        if(m2 != j2){
+            F77_CALL(dcopy)(&mm, Phi_new, INC_SEQ, Phi_tmp, INC_SEQ);
+            F77_CALL(dcopy)(&mm, Im, INC_SEQ, Phi_new, INC_SEQ);
+            if(j2>1){
+                for (int i = 0; i < j2; i++){
+                    for (int i0 = i+1; i0 < j2; i0++){
+                        Phi_new[i + i0 * m] = Phi_tmp[index_nonzerocolumn[i] + index_nonzerocolumn[i0] * m];
+                        Phi_new[i0 + i * m] = Phi_tmp[index_nonzerocolumn[i0] + index_nonzerocolumn[i] * m];
+                        //Rprintf("Phinew_sort: %5f\n",Phi_new[i + i0 * m]);
+                    }
+                }
+            }
+            for (int i = 0; i < j2; i++){
+                for (int i0 = 0; i0 < j2_2; i0++){
+                    Phi_new[i + (i0+j2) * m] = Phi_tmp[index_nonzerocolumn[i] + index_zerocolumn[i0] * m];
+                    Phi_new[(i0+j2) + i * m] = Phi_tmp[index_zerocolumn[i0] + index_nonzerocolumn[i] * m];
+                        //Rprintf("Phinew_sort: %5f\n",Phi_new[(i0+j2) + i * m]);
+                }
+            }
+            if(j2_2>1){
+                for (int i = 0; i < j2_2; i++){
+                    for (int i0 = i+1; i0 < j2_2; i0++){
+                        Phi_new[(i+j2) + (i0+j2) * m] = Phi_tmp[index_zerocolumn[i] + index_zerocolumn[i0] * m];
+                        Phi_new[(i0+j2) + (i+j2) * m] = Phi_tmp[index_zerocolumn[i0] + index_zerocolumn[i] * m];
+                        //Rprintf("Phinew_sort: %5f\n",Phi_new[(i0+j2) + (i+j2) * m]);
+                    }
+                }
+            }
+        }
+        m2 = j2;
+
 		// Check convergence
 		err_Lambda = dnorm2(pm, Lambda_new, Lambda);
+		err_Lambda = err_Lambda / sqrt(pm); //changed the threshold!!
 		
 		// Step
 		SWAP(double*, Lambda, Lambda_new);
@@ -684,6 +890,7 @@ static void minimize_logF_Lambda_Psi_Phi(
 	}
 	O_conv[0] |= err_Lambda > ctrl->tol_EM;
 	
+
 	// Suppress almost zero to zero
 	for (int i = 0; i < mm; i++) {
 		if (fabs(Phi[i]) < ctrl->tol_Phi) Phi[i] = 0;
@@ -734,7 +941,7 @@ static void minimize_logF_Lambda_Psi_Phi(
 	// Calculate logF
 	struct logF_value_t logF;
 	calculate_logF(
-		p, m, N, Lambda, diag_Psi, Phi, S, X, Im, rho, gamma,
+		p, m, N, Lambda, diag_Psi, Phi, S, X, Im, rho, gamma, type,
 		cor_factor, ctrl, &logF);
 	
 	// Compose result
@@ -747,6 +954,8 @@ static void minimize_logF_Lambda_Psi_Phi(
 	vmaxset(vmax);
 }
 
+
+
 static void minimize_logF_Psi(
 	int p, int m, int N,
 	double *Lambda, double *diag_Psi_init, double *Phi,
@@ -799,8 +1008,8 @@ static void minimize_logF_Psi(
 // ---------------------------------------------------------
 
 static void init_params_warm(int p, int m, int N,
-	double *S, double *X, double *Im, struct ctrl_fanc_t *ctrl,
-	double *O_Lambda, double *O_diag_Psi, double *O_rho, int *conv)
+	double *S, double *X, double *Im, int type, struct ctrl_fanc_t *ctrl,
+	double *O_Lambda, double *O_diag_Psi, double *O_rho, int ISNAN_max_rho, int *conv)
 {
 	void *vmax = vmaxget();
 	int m2;
@@ -827,7 +1036,7 @@ static void init_params_warm(int p, int m, int N,
 		struct logF_value_t logF_tmp;
 		minimize_logF_Lambda_Psi_Phi(
 			p, 1, N, 1, Lambda_tmp, diag_Psi_tmp, Phi,
-			S, X, Im, 0, R_PosInf, 0, ctrl,
+			S, X, Im, 0, R_PosInf, type, 0, 0, ctrl,
 			&m2, Lambda_tmp, diag_Psi_tmp, Phi,
 			&logF_tmp, conv);
 		
@@ -873,15 +1082,167 @@ static void init_params_warm(int p, int m, int N,
 	dfill(p * m - p, O_Lambda + p, 0);
 	F77_CALL(dcopy)(&p, Lambda_opt, INC_SEQ, O_Lambda, INC_SEQ);
 	F77_CALL(dcopy)(&p, diag_Psi_opt, INC_SEQ, O_diag_Psi, INC_SEQ);
-	if (O_rho != NULL) *O_rho = max_rho;
+	if (ISNAN_max_rho) O_rho[0] = max_rho;
 	
 	vmaxset(vmax);
 }
 
+static void init_params_warm_prenet(int p, int m, int N,
+	double *S, double *X, double *Im, double *gammas, int type, int use_cor_fact, struct ctrl_fanc_t *ctrl,
+	int *O_m2, double *O_Lambda, double *O_diag_Psi, double *O_Phi,
+	double *O_rho, int ISNAN_max_rho, int *conv)
+{
+	int pm=p*m;
+	int mm=m*m;
+
+	void *vmax = vmaxget();
+	int m2, m2_opt;
+	double *Lambda_opt = (double *)R_alloc(pm, sizeof(double));
+	double *Lambda_tmp = (double *)R_alloc(pm, sizeof(double));
+	double *Lambda_opt_2factor = (double *)R_alloc(pm, sizeof(double));
+	double *Lambda_tmp_2factor = (double *)R_alloc(pm, sizeof(double));
+	double *diag_Psi_opt = (double *)R_alloc(p, sizeof(double));
+	double *diag_Psi_tmp = (double *)R_alloc(p, sizeof(double));
+	double *diag_Psi_opt_2factor = (double *)R_alloc(p, sizeof(double));
+	double *diag_Psi_tmp_2factor = (double *)R_alloc(p, sizeof(double));
+	double *Phi_new = (double *)R_alloc(mm, sizeof(double));
+	double *Phi_tmp = (double *)R_alloc(mm, sizeof(double));
+	double *Phi_opt = (double *)R_alloc(mm, sizeof(double));
+	double *Phi_tmp_2factor = (double *)R_alloc(mm, sizeof(double));
+	double *Phi_opt_2factor = (double *)R_alloc(mm, sizeof(double));
+	double *M = (double *)R_alloc(mm, sizeof(double));
+	double *A = (double *)R_alloc(mm, sizeof(double));
+	double *B = (double *)R_alloc(pm, sizeof(double));
+	double min_logF_pen = DBL_MAX;
+	double min_logF_pen_2factor = DBL_MAX;
+
+    int *index_nonzerorow = (int *)R_alloc(p, sizeof(int));
+	
+    F77_CALL(dcopy)(&mm, Im, INC_SEQ, Phi_new, INC_SEQ);
+
+	// Search appropriate Lambda
+	for (int t = 0; t < ctrl->num_iter_init_prenet; t++) {
+//	for (int t = 0; t < 100; t++) {//ここ変更！！
+		// Make parameter candidate at random
+		for (int i = 0; i < pm; i++) {
+			Lambda_tmp[i] = unif_rand();
+			Lambda_tmp_2factor[i] = unif_rand();
+		}
+		for (int i = 0; i < p; i++) {
+			diag_Psi_tmp[i] = unif_rand();
+			diag_Psi_tmp_2factor[i] = unif_rand();
+		}
+    	F77_CALL(dcopy)(&mm, Im, INC_SEQ, Phi_tmp, INC_SEQ);
+/*
+	if(t==0){
+		for (int i = 0; i < p; i++) {
+			for (int j = 0; j < m; j++){
+				Rprintf("%5f ", Lambda_tmp[i+j*p]);
+			}
+			Rprintf("\n");
+		}
+		for (int j = 0; j < p; j++){
+			Rprintf("%5f ", diag_Psi_tmp[j]);
+		}
+
+	}		
+		*/
+
+		// Get better candidate from random parameter 
+		struct logF_value_t logF_tmp;
+		// fit uncorrelated model	
+		minimize_logF_Lambda_Psi_Phi(
+			p, m, N, m, Lambda_tmp, diag_Psi_tmp, Phi_tmp,
+			S, X, Im, 0.0, 1.0, type, 1, 0, ctrl,
+			&m2, Lambda_tmp, diag_Psi_tmp, Phi_tmp,
+			&logF_tmp, conv);
+		// fit correlated model	
+		if(use_cor_fact) minimize_logF_Lambda_Psi_Phi(
+			p, m, N, m, Lambda_tmp, diag_Psi_tmp, Phi_tmp,
+			S, X, Im, 0.0, 1.0, type, 1, use_cor_fact, ctrl,
+			&m2, Lambda_tmp, diag_Psi_tmp, Phi_tmp,
+			&logF_tmp, conv);
+    	if(m2>1){
+    		F77_CALL(dcopy)(&pm, Lambda_tmp, INC_SEQ, Lambda_tmp_2factor, INC_SEQ);
+	    	F77_CALL(dcopy)(&p, diag_Psi_tmp, INC_SEQ, diag_Psi_tmp_2factor, INC_SEQ);
+	    	F77_CALL(dcopy)(&mm, Phi_tmp, INC_SEQ, Phi_tmp_2factor, INC_SEQ);
+	    }
+		
+		// Update if better
+		if (logF_tmp.logF_pen < min_logF_pen) {
+			SWAP(double*, Lambda_tmp, Lambda_opt);
+			SWAP(double*, diag_Psi_tmp, diag_Psi_opt);
+			SWAP(double*, Phi_tmp, Phi_opt);
+			min_logF_pen = logF_tmp.logF_pen;
+			m2_opt = m2;
+		}
+		
+
+		if (logF_tmp.logF_pen < min_logF_pen_2factor && m2>1) {
+			SWAP(double*, Lambda_tmp_2factor, Lambda_opt_2factor);
+			SWAP(double*, diag_Psi_tmp_2factor, diag_Psi_opt_2factor);
+			SWAP(double*, Phi_tmp_2factor, Phi_opt_2factor);
+			min_logF_pen_2factor = logF_tmp.logF_pen;
+		}
+
+	}
+	int n_gam = ctrl->num_gammas;
+	double *max_rho = (double *)R_alloc(n_gam, sizeof(double));
+	
+    double cand_rho = 0.0;
+    dfill(n_gam, max_rho, 0.0);
+    //nonzeroindex <- apply(Lambda,1,function(x) which(x!=0))
+    for (int i = 0; i < p; i++) {
+        for (int j = 0; j < m; j++) {
+            if(Lambda_opt_2factor[i+j*p] != 0.0){
+                index_nonzerorow[i] = j;
+            }
+        }
+    }
+
+	calculate_M_A_B(
+			p, m, N, Lambda_opt_2factor, diag_Psi_opt_2factor, Phi_opt_2factor, S, X, Im, ctrl->cor_factor, ctrl,
+			M, A, B);
+
+	for(int l = 0; l < n_gam; l++){
+        for (int i = 0; i < p; i++) {
+            int j = index_nonzerorow[i];
+            for (int k = 0; k < m; k++) {
+                if(j != k){
+                //rho_cand <- (B[k,i] - A[k,j] * Lambda_i[i,j]) / ( alpha * diagPsi_i[i] * Lambda_i[i,j] )
+                    cand_rho = (B[k+i*m] - A[k+j*m] * Lambda_opt_2factor[i+j*p]) / ( gammas[l] * diag_Psi_opt_2factor[i] * Lambda_opt_2factor[i+j*p] * ctrl->w[i] );
+                //rho_cand <- abs(rho_cand)
+                    cand_rho = fabs(cand_rho);
+                    if(max_rho[l] < cand_rho){
+                        max_rho[l] = cand_rho;
+                    }
+                }
+            }
+        }
+    }
+    	//Rprintf("%5f\n",max_rho[0]);
+    	//Rprintf("%5f\n",max_rho[1]);
+    	//Rprintf("%5f\n",max_rho[2]);
+
+
+    //slightly enlarge the maximum value of rho
+	for(int l = 0; l < n_gam; l++) max_rho[l] = max_rho[l] +  max_rho[l] * ctrl->delta;
+	F77_CALL(dcopy)(&pm, Lambda_opt, INC_SEQ, O_Lambda, INC_SEQ);
+	F77_CALL(dcopy)(&p, diag_Psi_opt, INC_SEQ, O_diag_Psi, INC_SEQ);
+	F77_CALL(dcopy)(&mm, Phi_opt, INC_SEQ, O_Phi, INC_SEQ);
+	if (ISNAN_max_rho)  F77_CALL(dcopy)(&n_gam, max_rho, INC_SEQ, O_rho, INC_SEQ);
+
+	O_m2[0]=m2_opt;
+
+	
+	vmaxset(vmax);
+}
+
+
 static void init_params_random(
 	int p, int m, int N, int m2,
 	double *S, double *X, double *Im,
-	double rho, double gamma, int use_cor_fact, struct ctrl_fanc_t *ctrl,
+	double rho, double gamma, int type, int use_cor_fact, struct ctrl_fanc_t *ctrl,
 	int *O_m2, double *O_Lambda, double *O_diag_Psi, double *O_Phi, int *O_conv)
 {
 	void *vmax = vmaxget();
@@ -908,13 +1269,21 @@ static void init_params_random(
 		
 		// Get better candidate from random parameter
 		struct logF_value_t logF_tmp;
+		// First, fit the model with UNCORRELATED factors
 		minimize_logF_Lambda_Psi_Phi(
 			p, m, N, m2, Lambda_tmp, diag_Psi_tmp, Phi_tmp, S, X, Im,
-			rho, gamma, use_cor_fact, ctrl,
+			rho, gamma, type, 0, 0, ctrl,
 			&m2_tmp, Lambda_tmp, diag_Psi_tmp, Phi_tmp,
 			&logF_tmp, O_conv);
-		
-		// Update if better
+		// If use_cor_fact == 1, fit the model with CORRELATED factors
+		if(use_cor_fact == 1){
+			minimize_logF_Lambda_Psi_Phi(
+			p, m, N, m2, Lambda_tmp, diag_Psi_tmp, Phi_tmp, S, X, Im,
+			rho, gamma, type, 0, use_cor_fact, ctrl,
+			&m2_tmp, Lambda_tmp, diag_Psi_tmp, Phi_tmp,
+			&logF_tmp, O_conv);
+		}
+		// Update parameter if it is better 
 		if (logF_tmp.logF_pen < min_logF_pen) {
 			m2_opt = m2_tmp;
 			SWAP(double*, Lambda_tmp, Lambda_opt);
@@ -1060,6 +1429,7 @@ static void fanc(
 	int p, int m, int N, double *S, double *X, struct ctrl_fanc_t *ctrl,
 	double *O_rhos, double *O_gammas, int *O_m2,
 	int *O_spn_Lambda, int **O_spi_Lambda, double **O_spv_Lambda,
+	int *type, //type=1 -> MC, type=2 -> prenet
 	double *O_diag_Psi, double *O_Phi, double *O_logF,
 	double *O_df, double *O_BIC, double *O_BICH, double *O_EBIC,
 	double *O_AIC, double *O_CAIC, double *O_GFI, double *O_AGFI,
@@ -1078,6 +1448,7 @@ static void fanc(
 	double *diag_Psi_init = (double *)R_alloc(p, sizeof(double));
 	double *diag_Psi_full = (double *)R_alloc(p, sizeof(double));
 	double *Phi = (double *)R_alloc(mm, sizeof(double));
+	double *Phi_init = (double *)R_alloc(mm, sizeof(double));
 	double *Phi_full = (double *)R_alloc(mm, sizeof(double));
 	double *Im = (double *)R_alloc(mm, sizeof(double));
 	for (int i = 0; i < m; i++) {
@@ -1088,6 +1459,7 @@ static void fanc(
 	double *Sigma = (double *)R_alloc(pp, sizeof(double));
 	int num_rhos = ctrl->num_rhos, num_gammas = ctrl->num_gammas;
 	double rhos[num_rhos * num_gammas], gammas[num_gammas];
+	double rhos_tmp[num_rhos];
 	double df_base[num_rhos];
 	double logdet_S = 0.0;
 	double logdet_diagS = 0.0;
@@ -1099,41 +1471,112 @@ static void fanc(
 	}
 	double f0hat = logdet_diagS - logdet_S;
 
-	// Calculate initial parameters
-	double max_rho = ctrl->max_rho;
-	if (ISNAN(max_rho) || !ctrl->cold_init) {
-		init_params_warm(
-			p, m, N, S, X, Im, ctrl, Lambda_init, diag_Psi_init,
-			ISNAN(max_rho) ? &max_rho : NULL, O_conv);
+	int ISNAN_rhos = ISNAN(ctrl->rhos[0]);
+	int ISNAN_gammas = ISNAN(ctrl->gammas[0]);
+	int ISNAN_max_rho = ISNAN(ctrl->max_rho[0]);
+	double *max_rho = (double *)R_alloc(num_gammas, sizeof(double));
+	int m2_init_prenet;
+
+
+
+	// Determine gamma
+	if(ISNAN_gammas){
+		if(type[0]==1){
+			gammas[0] = R_PosInf;
+			logseq(num_gammas - 1, ctrl->max_gamma, ctrl->min_gamma, gammas + 1);
+		}else{
+			logseq(num_gammas, ctrl->max_gamma, ctrl->min_gamma, gammas);
+		}
+	}else{
+		F77_CALL(dcopy)(&num_gammas, ctrl->gammas, INC_SEQ, gammas, INC_SEQ);
 	}
-	if (max_rho > LARGE_RHO) {
+
+	//determine maxim value of rho
+	if(ISNAN_max_rho){
+		if(type[0]==1){
+			max_rho[0] = 0.0;
+		}
+		if(type[0]==2){
+			dfill(num_gammas, max_rho, 0.0);
+		}
+	}else{
+		if(type[0]==1) F77_CALL(dcopy)(&I_ONE, ctrl->max_rho, INC_SEQ, max_rho, INC_SEQ);
+		if(type[0]==2) F77_CALL(dcopy)(&num_gammas, ctrl->max_rho, INC_SEQ, max_rho, INC_SEQ);
+	}
+
+	// Calculate initial parameters
+	if (ISNAN_max_rho || !ctrl->cold_init) {
+		if(type[0]==1){
+			init_params_warm(
+			p, m, N, S, X, Im, type[0], ctrl, 
+			Lambda_init, diag_Psi_init, 
+			max_rho, ISNAN_max_rho, O_conv);
+			F77_CALL(dcopy)(&mm, Im, INC_SEQ, Phi_init, INC_SEQ);
+		}else{
+			init_params_warm_prenet(
+			p, m, N, S, X, Im, gammas, type[0], ctrl->cor_factor, ctrl, 
+			&m2, Lambda_init, diag_Psi_init, Phi_init,
+			max_rho, ISNAN_max_rho, O_conv);
+			m2_init_prenet = m2;
+		}
+
+	}
+
+
+/*
+	if(type[0]==2){
+		for (int i = 0; i < num_gammas; i++){
+			Rprintf("%5f ", max_rho[i]);
+		}
+	}
+*/
+	if (type[0]==1 && max_rho[0] > LARGE_RHO) {
 		warning(
 			"\"rho.max\" is greater than 8.25. In such cases, "
 			"the reparametrization of the penalty funcion may be failed");
 	}
 	if (!ctrl->cold_init) {
-		m2 = 1;
+		if(type[0]==1) m2 = 1;
+		if(type[0]==2) m2 = m2_init_prenet;
 		F77_CALL(dcopy)(&pm, Lambda_init, INC_SEQ, Lambda, INC_SEQ);
 		F77_CALL(dcopy)(&p, diag_Psi_init, INC_SEQ, diag_Psi, INC_SEQ);
-		F77_CALL(dcopy)(&mm, Im, INC_SEQ, Phi, INC_SEQ);
+		F77_CALL(dcopy)(&mm, Phi_init, INC_SEQ, Phi, INC_SEQ);
 	}
 	
-	// Determine rho and gamma
-	gammas[0] = R_PosInf;
-	logseq(num_gammas - 1, ctrl->max_gamma, ctrl->min_gamma, gammas + 1);
-	logseq(num_rhos, max_rho, max_rho * ctrl->delta, rhos);
-	for (int gi = 1; gi < num_gammas; gi++) {
-		for (int ri = 0; ri < num_rhos; ri++) {
-			rhos[ri + gi * num_rhos] = zeroin(
-				rhos[ri], dmax(max_rho, 8.3), zeroinfn_rho, DBL_EPSILON,
-				&(struct ex_zeroin_rho_t){gammas[gi], rhos[ri]});
-		}
-	}
-	if (ctrl->zero_min_rho) {
-		for (int gi = 0; gi < num_gammas; gi++) {
-			rhos[(num_rhos - 1) + gi * num_rhos] = 0;
+	// Determine rho
+	if(ISNAN_rhos){
+		if(type[0] == 1){
+			logseq(num_rhos, max_rho[0], max_rho[0] * ctrl->delta, rhos);
+			F77_CALL(dcopy)(&num_rhos, rhos, INC_SEQ, rhos_tmp, INC_SEQ);
+			int gimin;
+			if(gammas[0]==R_PosInf){
+				gimin = 1;
+			}else{
+				gimin = 0;
+			}
+			for (int gi = gimin; gi < num_gammas; gi++) {
+				for (int ri = 0; ri < num_rhos; ri++) {
+					rhos[ri + gi * num_rhos] = zeroin(
+						rhos_tmp[ri], dmax(max_rho[0], 8.3), zeroinfn_rho, DBL_EPSILON,
+						&(struct ex_zeroin_rho_t){gammas[gi], rhos_tmp[ri]});
+				}
+			}
+			if (ctrl->zero_min_rho) {
+				for (int gi = 0; gi < num_gammas; gi++) {
+					rhos[(num_rhos - 1) + gi * num_rhos] = 0;
+				}
+			}			
+		}else{
+			for (int gi = 0; gi < num_gammas; gi++) {
+				if(type[0]==1) logseq(num_rhos, max_rho[gi], max_rho[gi] * ctrl->delta, rhos + num_rhos * gi);
+				if(type[0]==2) powerseq(num_rhos, max_rho[gi], max_rho[gi] * ctrl->delta * sqrt(gammas[gi]), ctrl->alpha_powerseq, rhos + num_rhos * gi);
+			}
 		}
+	}else{
+		int num_gammas_rhos = num_rhos * num_gammas;
+		F77_CALL(dcopy)(&num_gammas_rhos, ctrl->rhos, INC_SEQ, rhos, INC_SEQ);
 	}
+
 	memcpy(O_rhos, rhos, num_rhos * num_gammas * sizeof(double));
 	memcpy(O_gammas, gammas, num_gammas * sizeof(double));
 	
@@ -1152,10 +1595,10 @@ static void fanc(
 		if (ctrl->cold_init) {
 			init_params_random(
 				p, m, N, m, S, X, Im,
-				rhos[rgi], gammas[gi], ctrl->cor_factor, ctrl,
+				rhos[rgi], gammas[gi], type[0], ctrl->cor_factor, ctrl,
 				&m2, Lambda, diag_Psi, Phi, conv);
 		} else {
-			if (gi > 0) {
+			if (gi > 0 && type[0]==1) {
 				// Init with (ri, gi-1) result when gi > 0
 				m2 = O_m2[ri + (gi - 1) * num_rhos];
 				dfill(pm, Lambda, 0);
@@ -1171,48 +1614,44 @@ static void fanc(
 				F77_CALL(dcopy)(
 					&mm, O_Phi + (ri + (gi - 1) * num_rhos) * mm,
 					INC_SEQ, Phi, INC_SEQ);
-			} else if (F77_CALL(dnrm2)(&pm, Lambda, &I_ONE) < ctrl->tol_EM) {
-				// Reinitialize when Lambda is zero matrix
-				m2 = 1;
+			} else if (ri == 0 && type[0]==2) {
 				F77_CALL(dcopy)(&pm, Lambda_init, INC_SEQ, Lambda, INC_SEQ);
 				F77_CALL(dcopy)(&p, diag_Psi_init, INC_SEQ, diag_Psi, INC_SEQ);
-				F77_CALL(dcopy)(&mm, Im, INC_SEQ, Phi, INC_SEQ);
+				F77_CALL(dcopy)(&mm, Phi_init, INC_SEQ, Phi, INC_SEQ);
+			}else if (F77_CALL(dnrm2)(&pm, Lambda, &I_ONE) < ctrl->tol_EM) {
+				// Reinitialize when Lambda is zero matrix
+				if(type[0]==1) m2 = 1;
+				if(type[0]==2) m2 = m2_init_prenet;
+				F77_CALL(dcopy)(&pm, Lambda_init, INC_SEQ, Lambda, INC_SEQ);
+				F77_CALL(dcopy)(&p, diag_Psi_init, INC_SEQ, diag_Psi, INC_SEQ);
+				F77_CALL(dcopy)(&mm, Phi_init, INC_SEQ, Phi, INC_SEQ);
 			}
 		}
+
 		
 		// Minimize with restricted model
 		struct logF_value_t logF;
 		minimize_logF_Lambda_Psi_Phi(
 			p, m, N, m2, Lambda, diag_Psi, Phi,
-			S, X, Im, rhos[rgi], gammas[gi], ctrl->cor_factor, ctrl,
+			S, X, Im, rhos[rgi], gammas[gi], type[0], 0, ctrl->cor_factor, ctrl,
 			&m2, Lambda, diag_Psi, Phi, &logF, conv);
 		
 		// Check if the number of factors should be extended
-		if (gi == 0 && m2 < m) {
+		if (m2 < m && ((type[0]==1 && gi == 0) || type[0]==2)) {
 			int m2_full;
 			struct logF_value_t logF_full;
 			
 			// Minimize with the full model
 			init_params_random(
-				p, m, N, m, S, X, Im, rhos[rgi], gammas[gi], 0, ctrl,
+				p, m, N, m, S, X, Im, rhos[rgi], gammas[gi], type[0], ctrl->cor_factor, ctrl,
 				&m2_full, Lambda_full, diag_Psi_full, Phi_full, conv);
 			minimize_logF_Lambda_Psi_Phi(
 				p, m, N, m, Lambda_full, diag_Psi_full, Phi_full,
-				S, X, Im, rhos[rgi], gammas[gi], 0, ctrl,
+				S, X, Im, rhos[rgi], gammas[gi], type[0], 0, ctrl->cor_factor, ctrl,
 				&m2_full, Lambda_full, diag_Psi_full, Phi_full,
 				&logF_full, conv);
 			
-			// Calculate precious value if the full model seems better
-			if (ctrl->cor_factor &&
-				m2_full > m2 && logF_full.logF_pen < logF.logF_pen)
-				{
-					minimize_logF_Lambda_Psi_Phi(
-						p, m, N, m, Lambda_full, diag_Psi_full, Phi_full,
-						S, X, Im, rhos[rgi], gammas[gi], 1, ctrl,
-						&m2_full, Lambda_full, diag_Psi_full, Phi_full,
-						&logF_full, conv);
-				}
-			
+				
 			// Update model
 			if (logF_full.logF_pen < logF.logF_pen) {
 				m2 = m2_full;
@@ -1223,13 +1662,12 @@ static void fanc(
 			}
 		}
 		
+		
+		// Compose result
 		int nonzero = 0;
 		for (int i = 0; i < pm; i++) {
 			if (Lambda[i] != 0) nonzero++;
 		}
-		if (gi == 0) df_base[ri] = p + nonzero;
-		
-		// Compose result
 		O_m2[rgi] = m2;
 		O_spn_Lambda[rgi] = nonzero;
 		O_spi_Lambda[rgi] = (int *)malloc(nonzero * sizeof(int));
@@ -1316,7 +1754,7 @@ static void set_list_element(SEXP list, int i, const char *key, SEXP val) {
 #define __REXTCALL_CLEAN__ UNPROTECT(nprot); nprot = 0;
 
 SEXP RextCall_fanc(
-	SEXP R_p, SEXP R_m, SEXP R_N, SEXP R_S, SEXP R_X, SEXP R_ctrl)
+	SEXP R_p, SEXP R_m, SEXP R_N, SEXP R_S, SEXP R_X, SEXP R_type, SEXP R_ctrl)
 {
 	__REXTCALL_INIT__
 	GetRNGstate();
@@ -1332,12 +1770,16 @@ SEXP RextCall_fanc(
 		REAL(get_list_element(R_ctrl, "zita"))[0],
 		REAL(get_list_element(R_ctrl, "Delta"))[0],
 		REAL(get_list_element(R_ctrl, "init.coef")),
-		REAL(get_list_element(R_ctrl, "rho.max"))[0],
+		REAL(get_list_element(R_ctrl, "rho")),
+		REAL(get_list_element(R_ctrl, "gamma")),
+		REAL(get_list_element(R_ctrl, "max.rho")),
 		REAL(get_list_element(R_ctrl, "max.gamma"))[0],
 		REAL(get_list_element(R_ctrl, "min.gamma"))[0],
 		REAL(get_list_element(R_ctrl, "gamma.ebic"))[0],
+		REAL(get_list_element(R_ctrl, "w")),
 		INTEGER(get_list_element(R_ctrl, "length.rho"))[0],
 		INTEGER(get_list_element(R_ctrl, "length.gamma"))[0],
+		REAL(get_list_element(R_ctrl, "alpha.powerseq"))[0],
 		INTEGER(get_list_element(R_ctrl, "maxit.em"))[0],
 		INTEGER(get_list_element(R_ctrl, "maxit.cd"))[0],
 		INTEGER(get_list_element(R_ctrl, "maxit.bfgs"))[0],
@@ -1347,6 +1789,7 @@ SEXP RextCall_fanc(
 		strcmp(CHAR(STRING_ELT(get_list_element(R_ctrl, "start"), 0)), "cold")
 		== 0,
 		INTEGER(get_list_element(R_ctrl, "ncand.initial"))[0],
+		INTEGER(get_list_element(R_ctrl, "ncand.initial.prenet"))[0],
 		INTEGER(get_list_element(R_ctrl, "pmax_for_S"))[0],
 		INTEGER(get_list_element(R_ctrl, "progress"))[0],
 		INTEGER(get_list_element(R_ctrl, "openmp"))[0],
@@ -1403,7 +1846,7 @@ SEXP RextCall_fanc(
 		INTEGER(get_list_element(R_ctrl, "cor.factor"))[0];
 	REXTCALL_DEF(R_Npflag, allocVector(LGLSXP, 1));
 	INTEGER(R_Npflag)[0] = N >= p;
-	REXTCALL_DEF(R_conv, allocVector(INTSXP, 3 * num_rhos * num_gammas));
+	REXTCALL_DEF(R_conv, allocVector(INTSXP, 3 * num_rhos * num_gammas)); //R_conv: convergence
 	REXTCALL_DEF(R_dim_conv, allocVector(INTSXP, 3));
 	INTEGER(R_dim_conv)[0] = 3;
 	INTEGER(R_dim_conv)[1] = num_rhos;
@@ -1412,12 +1855,13 @@ SEXP RextCall_fanc(
 	
 	int *spi_Lambda[num_rhos * num_gammas];
 	double *spv_Lambda[num_rhos * num_gammas];
-	pfill(num_rhos * num_gammas, (void **)spi_Lambda, NULL);
+	pfill(num_rhos * num_gammas, (void **)spi_Lambda, NULL); //fill spi_Lambda by NULL
 	pfill(num_rhos * num_gammas, (void **)spv_Lambda, NULL);
 	
 	fanc(p, m, N, REAL(R_S), REAL(R_X), &ctrl, 
 		REAL(R_rhos), REAL(R_gammas), INTEGER(R_m2), 
 		INTEGER(R_spn_Lambda), spi_Lambda, spv_Lambda, 
+		INTEGER(R_type), //type=1 -> MC, type=2 -> prenet
 		REAL(R_diag_Psi), REAL(R_Phi), REAL(R_logF), 
 		REAL(R_df), REAL(R_BIC), REAL(R_BICH), REAL(R_EBIC),
 		REAL(R_AIC), REAL(R_CAIC), REAL(R_GFI), REAL(R_AGFI),
